import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.*;

public class XMLParse extends DefaultHandler {

    public static void main(String argv[]) {
        if (argv.length != 1) {
            System.err.println("Usage: XMLParse filename");
            System.exit(1);
        }
        DefaultHandler handler = new XMLParse();
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try {
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse( new File(argv[0]), handler );
        }
        catch (ParserConfigurationException ex) {
            System.err.println ("Failed to create SAX parser:" + ex);
        }
        catch (SAXException ex) {
            System.err.println ("SAX parser exeception:" + ex);
        }
        catch (IOException ex) {
            System.err.println ("IO exeception:" + ex);
        }
        catch (IllegalArgumentException ex) {
            System.err.println ("Invalid file argument" + ex);
        }
    }
    public void startDocument() throws SAXException {
        System.out.println ("START DOCUMENT");
    }

    public void endDocument() throws SAXException {
        System.out.println ("END DOCUMENT");
    }

    public void startElement(String uri, String localName, String qualifiedName, Attributes attributes)
                  throws SAXException {
        System.out.println ("START ELEMENT " + qualifiedName);
        for (int i = 0; i< attributes.getLength(); i++) {
            System.out.println ("ATTRIBUTE " + attributes.getQName(i) + " = " + attributes.getValue(i));
        }
    }

    public void endElement(String uri, String localName, String qualifiedName) throws SAXException {
       System.out.println ("END ELEMENT " + qualifiedName);
    }

    public void characters(char[] ch, int start, int length) throws SAXException {
        if (length > 0) {
            String buf = new String (ch, start, length);
            System.out.println ("CONTENT " + buf);
        }
    }

    public void processingInstruction(String target, String data) throws SAXException {
        System.out.println ("PI " + target + " " + data);
    }

    public void notationDecl(String name, String publicId, String systemId) throws SAXException {
        System.out.println ("DECL " + name + " " + publicId + " " + systemId);
    }
}

