package agency;

import java.rmi.*;
import java.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.* ;
import javax.sql.*;

import data.*;

public class RegisterBean implements SessionBean
{
	private ApplicantLocalHome applicantHome;
	private SkillLocalHome skillHome;
	private LocationLocalHome locationHome;

	private ApplicantLocal applicant;

	public void updateDetails (ApplicantValueObject details)
    {
        if (details.getSkills() == null) {
            details.setSkills(new String[0]);
        }
        List skillList;
        try {
            skillList = skillHome.lookup(Arrays.asList(details.getSkills()));
        } catch(FinderException ex) {
            error("Invalid skill", ex); // throws an exception
            return;
        }

        LocationLocal location = null;
		if (details.getLocation() != null) {
	        try {
	            location = locationHome.findByPrimaryKey(details.getLocation());
	        } catch(FinderException ex) {
	            error("Invalid location", ex); // throws an exception
	            return;
	        }
		}

        applicant.setName(details.getName());
		applicant.setEmail(details.getEmail());
		applicant.setLocation(location);
		applicant.setSummary(details.getSummary());
		applicant.setSkills(skillList );

	}

	public ApplicantValueObject getDetails()
    {
		Collection col = applicant.getSkills();
		String[] skills = new String[col.size()];
		int i = 0;
		for(Iterator iter = col.iterator(); iter.hasNext(); ) {
			Object o = iter.next();
			SkillLocal skill = (SkillLocal)o;
			skills[i++] = skill.getName();
		}

		LocationLocal location = applicant.getLocation();
		String locationName = (location!=null)?location.getName():null;

	    ApplicantValueObject details = new ApplicantValueObject();
		details.setLogin(applicant.getLogin());
		details.setName(applicant.getName());
		details.setEmail(applicant.getEmail());
		details.setLocation(locationName);
		details.setSummary(applicant.getSummary());

		details.setSkills(skills);

		return details;
	}

	private void error (String msg, Exception ex) {
		String s = "RegisterBean: "+msg + "\n" + ex;
		System.out.println(s);
		throw new EJBException(s,ex);
	}

	// EJB methods start here

	public void ejbCreate (String login) throws CreateException {
		try {
			applicant = applicantHome.findByPrimaryKey(login);
		}
		catch (FinderException ex) {
			error ("Cannot find applicant: "+login,ex);
		}
	}

	public void ejbActivate(){
	}

	public void ejbPassivate(){
	}

	public void ejbRemove(){
		applicant = null;
		applicantHome = null;
	}

	private SessionContext ctx;

	public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
		InitialContext ic = null;
		try {
			ic = new InitialContext();
			applicantHome = (ApplicantLocalHome)ic.lookup("java:comp/env/ejb/ApplicantLocal");
			skillHome = (SkillLocalHome)ic.lookup("java:comp/env/ejb/SkillLocal");
			locationHome = (LocationLocalHome)ic.lookup("java:comp/env/ejb/LocationLocal");
		}
		catch (NamingException ex) {
			error("Error looking up depended EJB or resource",ex);
		}
	}
}

