package agency;

import java.rmi.*;
import java.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.* ;
import javax.sql.*;

public class AdvertiseBean implements SessionBean
{
  private JobDAO jobDAO;
    private DataSource dataSource;

	private String login;
	private String name;
	private String email;
	private String[] address;
		
	public void updateDetails (String name, String email, String[] address) {
		String[] dbAddress = {"",""};
		for (int i=0; i<Math.min(dbAddress.length,address.length); i++)
			dbAddress[i] = address[i];
			
		Connection con = null;
        PreparedStatement stmt = null;
        try {
            con = dataSource.getConnection();
            stmt = con.prepareStatement(
            "UPDATE Customer SET name = ?, email = ?, address1 = ?, address2 = ? WHERE login = ?");

            stmt.setString(1, name);
            stmt.setString(2, email);
            stmt.setString(3, dbAddress[0]);
            stmt.setString(4, dbAddress[1]);
            stmt.setString(5, login);
            stmt.executeUpdate();
            
            this.name = name;
            this.email = email;
            this.address = dbAddress;
        }
        catch (SQLException e) {
        	try
        	{
	        	if (con != null)
		        	con.rollback();
        	}
        	catch (SQLException ex) {}
            error("Error updating "+login,e);
		}
        finally {
            closeConnection(con, stmt, null);
        }
	}
	
	public String getLogin() {
		return login;
	}

	public String getName() {
		return name;
	}

	public String getEmail() {
		return email;
	}

	public String[] getAddress() {
		return address;
	}
	
	public String[] getJobs()
        {
		try
		{
			Collection col = jobDAO.findByCustomer(login);
			String[] res = new String[col.size()];
			Iterator it = col.iterator();
			int i = 0;
			while (it.hasNext())
			{
				JobValueObject job = (JobValueObject)it.next();
				res[i++] = job.getRef();
			}
			return (res);
		}
		catch (Exception ex) {
			error("Cannot find jobs for customer: "+login, ex);
		}
		return null;
	}

	public void createJob (String ref) throws DuplicateException, CreateException
        {
          try
	  {
	    jobDAO.createJob(ref, login);
	  }
	  catch(Exception ex)
	  {
	    error("Could not create job for " + ref, ex);
	  }
	}

	public void deleteJob (String ref) throws NotFoundException
        {
	  try
	  {
	    jobDAO.deleteJob(ref, login);
	  }
	  catch (Exception ex)
	  {
	    error("Cannot remove job: " + ref, ex);
	  }	  
	}

	private void error (String msg, Exception ex) {
		String s = "AdvertiseBean: "+msg + "\n" + ex;
        System.out.println(s);
        throw new EJBException(s, ex);
	}

  private void closeConnection(Connection con, PreparedStatement stmt, ResultSet rslt)
  {
    // DUMMY FOR COMPILATION
  }
	// EJB methods start here
	
	public void ejbCreate (String login) throws CreateException {
		this.login = login;
        Connection con = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            con = dataSource.getConnection();
            stmt = con.prepareStatement(
            "SELECT name,email,address1,address2 FROM Customer WHERE login = ?");

            stmt.setString(1, login);
            rs = stmt.executeQuery();

            if (rs.next()) {
                name = rs.getString(1);
                email = rs.getString(2);
                address = new String[2];
                address[0] = rs.getString(3);
                address[1] = rs.getString(4);
            }
            else
            	throw new CreateException ("Create failed to find databse record for "+login);

        }
        catch (SQLException e) {
            error("Error creating "+login,e);
		}
        finally {
            closeConnection(con, stmt, rs);
        }
	}

    public void ejbActivate(){
    }

    public void ejbPassivate(){
    }

    public void ejbRemove(){
    }

    private SessionContext ctx;
    
    public void setSessionContext(SessionContext ctx) {
		this.ctx = ctx;
        try {
	        InitialContext ic = new InitialContext();
	        dataSource = (DataSource)ic.lookup("java:comp/env/jdbc/Agency");
        }
        catch (NamingException ex) {
            error("Error connecting to java:comp/env/jdbc/Agency:",ex);
        }
        try
	{
	  jobDAO = new DirectJobDAOImpl("java:comp/env/jdbc/Agency");
	}
	catch (Exception ex)
	{
	  error("Error obtaining JobDAO", ex);
	}
    }
}

