package agency;

import java.sql.*;
import javax.naming.*;
import java.util.*;

public class DirectJobDAOImpl extends DirectDAOImpl implements JobDAO
{
  public DirectJobDAOImpl(String jndiName) throws SQLException, NamingException
  {
    super(jndiName);  
  }

  public void createJob(String ref, String customer) throws Exception
  {
    PreparedStatement stmt = null;
    try
    {
      Connection connection = acquireConnection();
      stmt = connection.prepareStatement(
			"INSERT INTO Job (ref,customer) VALUES (?, ?)");

      stmt.setString(1, ref);
      stmt.setString(2, customer);

      stmt.executeUpdate();
    }
    catch (SQLException e)
    {
        error("Error creating Job "+ customer +":"+ref, e);
    }
    finally
    {
      releasePreparedStatement(stmt);
      releaseConnection();
    }
  }

  public void deleteJob (String ref, String customer) throws Exception
  {
    PreparedStatement stmt = null;
    Connection connection = null;
    try
    {
      connection = acquireConnection();
      connection.setAutoCommit(false);
      stmt = connection.prepareStatement(
            "DELETE FROM JobSkill WHERE job = ? AND customer = ?");
            
      stmt.setString(1, ref);
      stmt.setString(2, customer);
      stmt.executeUpdate();
            
      stmt = connection.prepareStatement(
            "DELETE FROM Job WHERE ref = ? AND customer = ?");

      stmt.setString(1, ref);
      stmt.setString(2, customer);
      stmt.executeUpdate();
      connection.commit();
    }
    catch (SQLException e)
    {
      try
      {
        if (connection != null)
	{
          connection.rollback();
        }
      }
      catch (SQLException ex) {}
      error("Error deleting job "+ref+" for "+customer, e);
    }
    finally
    {
      releasePreparedStatement(stmt);
      releaseConnection();
    }
  }

  public Collection findByCustomer(String customer) throws Exception
  {
    PreparedStatement stmt = null;
    ResultSet rs = null;
    Collection jobs = new TreeSet();
    try
    {
      Connection connection = acquireConnection();
      stmt = connection.prepareStatement(
            "SELECT ref FROM Job WHERE customer = ?");

      stmt.setString(1, customer);
      rs = stmt.executeQuery();

      jobs.clear();
      while (rs.next())
      {
        jobs.add(rs.getString(1));
      }
    }
    catch (SQLException e)
    {
      error("Error loading jobs for "+customer, e);
    }
    finally
    {
      releasePreparedStatement(stmt);
      releaseResultSet(rs);
      releaseConnection();
    }
    return jobs;
  }

  private void error (String msg, Exception ex) throws Exception
  {
     String s = "DirectJobDAOImpl: "+msg + "\n" + ex;
     System.out.println(s);
     throw new Exception(s +  ex);
  }
}
